@@ -42,6 +42,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #endif
 
 #ifdef MACOS_X_ACCELERATION_HACK
+#include <IOKit/IOTypes.h>
 #include <IOKit/hidsystem/IOHIDLib.h>
 #include <IOKit/hidsystem/IOHIDParameter.h>
 #include <IOKit/hidsystem/event_status_driver.h>
@@ -135,7 +136,8 @@ static const char *IN_TranslateSDLToQ3Key(SDL_keysym *keysym, int *key)
 			case SDLK_RCTRL:        *key = K_CTRL;          break;
 
 			case SDLK_RMETA:
-			case SDLK_LMETA:
+			case SDLK_LMETA:        *key = K_COMMAND;       break;
+
 			case SDLK_RALT:
 			case SDLK_LALT:         *key = K_ALT;           break;
 
@@ -218,7 +220,7 @@ static void IN_PrintKey(const SDL_Event* event)
 IN_GetIOHandle
 ===============
 */
-static io_connect_t IN_GetIOHandle() // mac os x mouse accel hack
+static io_connect_t IN_GetIOHandle(void) // mac os x mouse accel hack
 {
 	io_connect_t iohandle = MACH_PORT_NULL;
 	kern_return_t status;
@@ -287,18 +289,17 @@ static void IN_ActivateMouse( void )
 
 	if( !mouseActive )
 	{
-		SDL_WM_GrabInput( SDL_GRAB_ON );
 		SDL_ShowCursor( 0 );
-
 #ifdef MACOS_X_CURSOR_HACK
 		// This is a bug in the current SDL/macosx...have to toggle it a few
 		//  times to get the cursor to hide.
 		SDL_ShowCursor( 1 );
 		SDL_ShowCursor( 0 );
 #endif
+		SDL_WM_GrabInput( SDL_GRAB_ON );
 	}
 
-	// in_nograb makes no sense unless fullscreen
+	// in_nograb makes no sense in fullscreen mode
 	if( !r_fullscreen->integer )
 	{
 		if( in_nograb->modified || !mouseActive )
@@ -322,7 +323,15 @@ IN_DeactivateMouse
 */
 static void IN_DeactivateMouse( void )
 {
-	if (!mouseAvailable || !SDL_WasInit( SDL_INIT_VIDEO ) )
+	if( !SDL_WasInit( SDL_INIT_VIDEO ) )
+		return;
+
+	// Always show the cursor when the mouse is disabled,
+	// but not when fullscreen
+	if( !r_fullscreen->integer )
+		SDL_ShowCursor( 1 );
+
+	if( !mouseAvailable )
 		return;
 
 #ifdef MACOS_X_ACCELERATION_HACK
@@ -346,8 +355,8 @@ static void IN_DeactivateMouse( void )
 
 	if( mouseActive )
 	{
-		SDL_ShowCursor( 1 );
 		SDL_WM_GrabInput( SDL_GRAB_OFF );
+		SDL_WarpMouse( glConfig.vidWidth / 2, glConfig.vidHeight / 2 );
 
 		mouseActive = qfalse;
 	}
@@ -396,10 +405,10 @@ struct
 
 /*
 ===============
-IN_StartupJoystick
+IN_InitJoystick
 ===============
 */
-static void IN_StartupJoystick( void )
+static void IN_InitJoystick( void )
 {
 	int i = 0;
 	int total = 0;
@@ -450,8 +459,22 @@ static void IN_StartupJoystick( void )
 	Com_DPrintf( "Balls: %d\n", SDL_JoystickNumBalls(stick) );
 
 	SDL_JoystickEventState(SDL_QUERY);
+}
 
-	return;
+/*
+===============
+IN_ShutdownJoystick
+===============
+*/
+static void IN_ShutdownJoystick( void )
+{
+	if (stick)
+	{
+		SDL_JoystickClose(stick);
+		stick = NULL;
+	}
+
+	SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
 }
 
 /*
@@ -651,7 +674,6 @@ static void IN_ProcessEvents( void )
 	SDL_Event e;
 	const char *p = NULL;
 	int key = 0;
-	int mx = 0, my = 0;
 
 	if( !SDL_WasInit( SDL_INIT_VIDEO ) )
 			return;
@@ -692,10 +714,7 @@ static void IN_ProcessEvents( void )
 
 			case SDL_MOUSEMOTION:
 				if (mouseActive)
-				{
-					mx += e.motion.xrel;
-					my += e.motion.yrel;
-				}
+					Com_QueueEvent( 0, SE_MOUSE, e.motion.xrel, e.motion.yrel, 0, NULL );
 				break;
 
 			case SDL_MOUSEBUTTONDOWN:
@@ -726,9 +745,6 @@ static void IN_ProcessEvents( void )
 				break;
 		}
 	}
-
-	if(mx || my)
-		Com_QueueEvent( 0, SE_MOUSE, mx, my, 0, NULL );
 }
 
 /*
@@ -740,8 +756,10 @@ void IN_Frame (void)
 {
 	IN_JoyMove( );
 
-	// Release the mouse if the console if down and we're windowed
-	if( ( Key_GetCatcher( ) & KEYCATCH_CONSOLE ) && !r_fullscreen->integer )
+	// Release the mouse if the console is down in windowed mode
+	// or if the window loses focus due to task switching
+	if( ( ( Key_GetCatcher( ) & KEYCATCH_CONSOLE ) && !r_fullscreen->integer ) ||
+	    !( SDL_GetAppState() & SDL_APPINPUTFOCUS ) )
 		IN_DeactivateMouse( );
 	else
 		IN_ActivateMouse( );
@@ -776,18 +794,22 @@ void IN_Init(void)
 	in_disablemacosxmouseaccel = Cvar_Get ("in_disablemacosxmouseaccel", "1", CVAR_ARCHIVE);
 #endif
 
-	Cvar_Set( "cl_platformSensitivity", "1.0" );
-
 	SDL_EnableUNICODE(1);
 	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
 	keyRepeatEnabled = qtrue;
 
-	if (in_mouse->value)
+	if( in_mouse->value )
+	{
 		mouseAvailable = qtrue;
+		IN_ActivateMouse( );
+	}
 	else
+	{
+		IN_DeactivateMouse( );
 		mouseAvailable = qfalse;
+	}
 
-	IN_StartupJoystick( );
+	IN_InitJoystick( );
 	Com_DPrintf ("------------------------------------\n");
 }
 
@@ -796,17 +818,21 @@ void IN_Init(void)
 IN_Shutdown
 ===============
 */
-void IN_Shutdown(void)
+void IN_Shutdown( void )
 {
-	IN_DeactivateMouse();
-
+	IN_DeactivateMouse( );
 	mouseAvailable = qfalse;
 
-	if (stick)
-	{
-		SDL_JoystickClose(stick);
-		stick = NULL;
-	}
+	IN_ShutdownJoystick( );
+}
 
-	SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
+/*
+===============
+IN_Restart
+===============
+*/
+void IN_Restart( void )
+{
+	IN_ShutdownJoystick( );
+	IN_Init( );
 }
